@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: waypoint.cxx,v 1.2 2004/01/31 19:50:35 curt Exp $
+// $Id: waypoint.cxx,v 1.3 2004/10/16 12:23:53 ehofman Exp $
 
 
 #include <simgear/math/polar3d.hxx>
@@ -29,12 +29,13 @@
 
 // Constructor
 SGWayPoint::SGWayPoint( const double lon, const double lat, const double alt,
-			const modetype m, const string s ) {
+			const modetype m, const string s, const string n ) {
     target_lon = lon;
     target_lat = lat;
     target_alt = alt;
     mode = m;
     id = s;
+    name = n;
 }
 
 
