@@ -2,7 +2,7 @@
 //
 // Written by Curtis Olson, started May 1998.
 //
-// Copyright (C) 1998  Curtis L. Olson  - curt@me.umn.edu
+// Copyright (C) 1998  Curtis L. Olson  - http://www.flightgear.org/~curt
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: matlib.cxx,v 1.12 2004/04/01 13:47:02 curt Exp $
+// $Id: matlib.cxx,v 1.14 2004/11/19 21:47:05 curt Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -33,12 +33,12 @@
 #  include <windows.h>
 #endif
 
-#include <GL/gl.h>
-
 #include <simgear/compiler.h>
 #include <simgear/constants.h>
 #include <simgear/structure/exception.hxx>
 
+#include SG_GL_H
+
 #include <string.h>
 #include STL_STRING
 
