@@ -2,7 +2,7 @@
 //
 // Written by Curtis Olson, started June 2000.
 //
-// Copyright (C) 2000  Curtis L. Olson  - curt@flightgear.org
+// Copyright (C) 2000  Curtis L. Olson  - http://www.flightgear.org/~curt
 //
 // This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,7 +13,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: cloud.cxx,v 1.32 2004/06/25 10:59:59 ehofman Exp $
+// $Id: cloud.cxx,v 1.39 2005/01/11 16:02:39 curt Exp $
 
 
 #include <simgear/compiler.h>
@@ -21,15 +21,13 @@
 // #include <stdio.h>
 #include <math.h>
 
-#if defined (__APPLE__) 
+#if defined (__APPLE__)
 // any C++ header file undefines isinf and isnan
 // so this should be included before <iostream>
 inline int (isinf)(double r) { return isinf(r); }
 inline int (isnan)(double r) { return isnan(r); } 
 #endif
 
-// #include STL_IOSTREAM
-
 #include <plib/sg.h>
 #include <plib/ssg.h>
 
@@ -47,6 +45,14 @@ inline int (isnan)(double r) { return isnan(r); }
 #define isnan(x) _isnan(x)
 #endif
 
+#if defined (__FreeBSD__)
+#  if __FreeBSD_version < 500000
+     extern "C" {
+       inline int isnan(double r) { return !(r <= 0 || r >= 0); }
+     }
+#  endif
+#endif
+
 
 static ssgStateSelector *layer_states[SGCloudLayer::SG_MAX_CLOUD_COVERAGES];
 static bool state_initialized = false;
@@ -191,9 +197,12 @@ generateNormalizationCubeMap()
 
 // Constructor
 SGCloudLayer::SGCloudLayer( const string &tex_path ) :
+    vertices(0),
+    indices(0),
     layer_root(new ssgRoot),
     layer_transform(new ssgTransform),
     state_sel(0),
+    cloud_alpha(1.0),
     texture_path(tex_path),
     layer_span(0.0),
     layer_asl(0.0),
@@ -204,9 +213,7 @@ SGCloudLayer::SGCloudLayer( const string &tex_path ) :
     speed(0.0),
     direction(0.0),
     last_lon(0.0),
-    last_lat(0.0),
-    vertices(0),
-    indices(0)
+    last_lat(0.0)
 {
     cl[0] = cl[1] = cl[2] = cl[3] = NULL;
     vl[0] = vl[1] = vl[2] = vl[3] = NULL;
@@ -489,7 +496,7 @@ SGCloudLayer::rebuild()
                            cos( j * half_angle ),
                            -sin( j * half_angle ) );
                 sgVectorProductVec3( v1.normal, v1.tTangent, v1.sTangent );
-                sgSetVec4( v1.color, 1.0f, 1.0f, 1.0f, (i == 0) ? 0.0f : 0.15f );
+                sgSetVec4( v1.color, 1.0f, 1.0f, 1.0f, (i == 0) ? 0.0f : cloud_alpha * 0.15f );
             }
         }
         /*
@@ -624,17 +631,33 @@ bool SGCloudLayer::repaint( sgVec3 fog_color ) {
         float *color;
 
         for ( int i = 0; i < 4; i++ ) {
-            for ( int j = 0; j < 10; ++j ) {
-                color = cl[i]->get( j );
+            color = cl[i]->get( 0 );
+            sgCopyVec3( color, fog_color );
+            color[3] = (i == 0) ? 0.0f : cloud_alpha * 0.15f;
+
+            for ( int j = 0; j < 4; ++j ) {
+                color = cl[i]->get( (2*j) + 1 );
                 sgCopyVec3( color, fog_color );
+                color[3] = 
+                    ((j == 0) || (i == 3)) ?
+                    ((j == 0) && (i == 3)) ? 0.0f : cloud_alpha * 0.15f : cloud_alpha;
+
+                color = cl[i]->get( (2*j) + 2 );
+                sgCopyVec3( color, fog_color );
+                color[3] = 
+                    ((j == 3) || (i == 0)) ?
+                    ((j == 3) && (i == 0)) ? 0.0f : cloud_alpha * 0.15f : cloud_alpha;
             }
+
+            color = cl[i]->get( 9 );
+            sgCopyVec3( color, fog_color );
+            color[3] = (i == 3) ? 0.0f : cloud_alpha * 0.15f;
         }
     }
 
     return true;
 }
 
-
 // reposition the cloud layer at the specified origin and orientation
 // lon specifies a rotation about the Z axis
 // lat specifies a rotation about the new Y axis
