@@ -33,9 +33,12 @@
 #  include <windows.h>
 #endif
 
-#include <GL/glu.h>
 #include <plib/ul.h>
 
+#include <simgear/compiler.h>
+
+#include SG_GLU_H
+
 #include "SkyCloud.hpp"
 #include "SkyRenderableInstance.hpp" 
 #include "SkyContext.hpp"
