@@ -30,7 +30,9 @@
 #  include <windows.h>
 #endif
 
-#include <GL/glu.h>
+#include <simgear/compiler.h>
+
+#include SG_GLU_H
 
 #include "SkyTextureManager.hpp"
 #include "SkyContext.hpp"
