@@ -3,7 +3,7 @@
 // Written by Curtis Olson, started December 1997.
 // SSG-ified by Curtis Olson, February 2000.
 //
-// Copyright (C) 1997-2000  Curtis L. Olson  - curt@flightgear.org
+// Copyright (C) 1997-2000  Curtis L. Olson  - http://www.flightgear.org/~curt
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 // Boston, MA  02111-1307, USA.
 //
-// $Id: dome.cxx,v 1.9 2004/02/18 14:33:16 ehofman Exp $
+// $Id: dome.cxx,v 1.11 2004/11/19 21:44:17 curt Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -33,7 +33,9 @@
 
 #include <math.h>
 
-#include <GL/gl.h>
+#include <simgear/compiler.h>
+
+#include SG_GL_H
 
 #include <plib/sg.h>
 
