@@ -3,7 +3,7 @@
 // Written by Curtis Olson, started December 1997.
 // SSG-ified by Curtis Olson, February 2000.
 //
-// Copyright (C) 1997-2000  Curtis L. Olson  - curt@flightgear.org
+// Copyright (C) 1997-2000  Curtis L. Olson  - http://www.flightgear.org/~curt
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 // Boston, MA  02111-1307, USA.
 //
-// $Id: sky.cxx,v 1.18 2004/08/17 08:28:35 ehofman Exp $
+// $Id: sky.cxx,v 1.22 2005/01/11 16:02:39 curt Exp $
 
 
 #include <plib/sg.h>
@@ -171,34 +171,34 @@ bool SGSky::reposition( SGSkyState &st, double dt )
 void SGSky::preDraw( float alt, float fog_exp2_density ) {
     ssgCullAndDraw( pre_root );
 
-    	// if we are closer than this to a cloud layer, don't draw clouds
+    // if we are closer than this to a cloud layer, don't draw clouds
     static const float slop = 5.0;
     int i;
 
     // check where we are relative to the cloud layers
     in_cloud = -1;
     for ( i = 0; i < (int)cloud_layers.size(); ++i ) {
-       float asl = cloud_layers[i]->getElevation_m();
-       float thickness = cloud_layers[i]->getThickness_m();
+        float asl = cloud_layers[i]->getElevation_m();
+        float thickness = cloud_layers[i]->getThickness_m();
 
-       if ( alt < asl - slop ) {
-           // below cloud layer
-       } else if ( alt < asl + thickness + slop ) {
-           // in cloud layer
+        if ( alt < asl - slop ) {
+            // below cloud layer
+        } else if ( alt < asl + thickness + slop ) {
+            // in cloud layer
 
-           // bail now and don't draw any clouds
-           in_cloud = i;
-       } else {
-           // above cloud layer
-       }
+            // bail now and don't draw any clouds
+            in_cloud = i;
+        } else {
+            // above cloud layer
+        }
     }
 
     // determine rendering order
     cur_layer_pos = 0;
     while ( cur_layer_pos < (int)cloud_layers.size() &&
-           alt > cloud_layers[cur_layer_pos]->getElevation_m())
+            alt > cloud_layers[cur_layer_pos]->getElevation_m() )
     {
-       ++cur_layer_pos;
+        ++cur_layer_pos;
     }
 
     // FIXME: This should not be needed, but at this time (08/15/2003)
@@ -267,7 +267,7 @@ void SGSky::modify_vis( float alt, float time_factor ) {
 	double ratio = 1.0;
 
         if ( cloud_layers[i]->getCoverage() == SGCloudLayer::SG_CLOUD_CLEAR ) {
-	    // clear layer
+	    // less than 50% coverage -- assume we're in the clear for now
 	    ratio = 1.0;
         } else if ( alt < asl - transition ) {
 	    // below cloud layer
@@ -286,9 +286,35 @@ void SGSky::modify_vis( float alt, float time_factor ) {
 	    ratio = 1.0;
 	}
 
-	// accumulate effects from multiple cloud layers
-	effvis *= ratio;
+        if ( cloud_layers[i]->getCoverage() == SGCloudLayer::SG_CLOUD_CLEAR ) {
+            // do nothing, clear layers aren't drawn, don't affect
+            // visibility andn dont' need to be faded in or out.
+        } else if ( (cloud_layers[i]->getCoverage() == 
+                     SGCloudLayer::SG_CLOUD_FEW)
+                    || (cloud_layers[i]->getCoverage() ==
+                        SGCloudLayer::SG_CLOUD_SCATTERED) )
+        {
+            // set the alpha fade value for the cloud layer.  For less
+            // dense cloud layers we fade the layer to nothing as we
+            // approach it because we stay clear visibility-wise as we
+            // pass through it.
+            float temp = ratio * 2.0;
+            if ( temp > 1.0 ) { temp = 1.0; }
+            cloud_layers[i]->setAlpha( temp );
 
+            // don't touch visibility
+        } else {
+            // maintain full alpha for denser cloud layer types.
+            // Let's set the value explicitly in case someone changed
+            // the layer type.
+            cloud_layers[i]->setAlpha( 1.0 );
+
+            // lower visibility as we approach the cloud layer.
+            // accumulate effects from multiple cloud layers
+            effvis *= ratio;
+        }
+
+#if 0
 	if ( ratio < 1.0 ) {
 	    if ( ! in_puff ) {
 		// calc chance of entering cloud puff
@@ -345,12 +371,14 @@ void SGSky::modify_vis( float alt, float time_factor ) {
 		    in_puff = false; 
 		}
 	    }
-
-	    // never let visibility drop below 25 meters
-	    if ( effvis <= 25.0 ) {
-		effvis = 25.0;
-	    }
 	}
+#endif
+
+        // never let visibility drop below 25 meters
+        if ( effvis <= 25.0 ) {
+            effvis = 25.0;
+        }
+
     } // for
 
     effective_visibility = effvis;
