@@ -2,7 +2,7 @@
 //
 // Written by Curtis Olson, started November 1998.
 //
-// Copyright (C) 1998  Curtis L. Olson - curt@flightgear.org
+// Copyright (C) 1998  Curtis L. Olson - http://www.flightgear.org/~curt
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -19,7 +19,7 @@
 // Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 // Boston, MA  02111-1307, USA.
 //
-// $Id: serial.cxx,v 1.5 2004/01/27 15:55:47 curt Exp $
+// $Id: serial.cxx,v 1.7 2004/12/05 09:36:49 ehofman Exp $
 
 
 #include <simgear/compiler.h>
@@ -129,7 +129,7 @@ bool SGSerialPort::open_port(const string& device) {
 
     // config.c_cflag |= CLOCAL;
 
-#if ! defined( sgi )    
+#if !defined( sgi ) && !defined(_AIX)
     // disable hardware flow control
     config.c_cflag &= ~(CRTSCTS);
 #endif
@@ -234,11 +234,11 @@ bool SGSerialPort::set_baud(int baud) {
 	speed = B19200;
     } else if ( baud == 38400 ) {
 	speed = B38400;
+#if defined( linux ) || defined( __FreeBSD__ )
     } else if ( baud == 57600 ) {
 	speed = B57600;
     } else if ( baud == 115200 ) {
 	speed = B115200;
-#if defined( linux ) || defined( __FreeBSD__ )
     } else if ( baud == 230400 ) {
 	speed = B230400;
 #endif
