@@ -2,7 +2,7 @@
 // 
 // Written by Curtis Olson, started April 2004.
 //
-// Copyright (C) 2004  Curtis L. Olson - curt@flightgear.org
+// Copyright (C) 2004  Curtis L. Olson - http://www.flightgear.org/~curt
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: sample_openal.cxx,v 1.14 2004/09/15 15:28:14 curt Exp $
+// $Id: sample_openal.cxx,v 1.16 2005/01/08 11:47:19 ehofman Exp $
 
 
 #if defined( __APPLE__ )
@@ -123,7 +123,7 @@ SGSoundSample::SGSoundSample( const char *path, const char *file,
     // Bind buffer with a source.
     alGenSources(1, &source);
     if (alGetError() != AL_NO_ERROR) {
-        throw sg_exception("Failed to gen source.");
+        throw sg_exception("Failed to gen source.\nPlease update your sound driver and try again.");
     }
 
     alSourcei( source, AL_BUFFER, buffer );
@@ -193,7 +193,7 @@ SGSoundSample::SGSoundSample( unsigned char *_data, int len, int _freq,
     // Bind buffer with a source.
     alGenSources(1, &source);
     if (alGetError() != AL_NO_ERROR) {
-        throw sg_exception("Failed to gen source.");
+        throw sg_exception("Failed to gen source.\nPlease update your sound driver and try again.");
     }
 
     alSourcei( source, AL_BUFFER, buffer );
