@@ -3,7 +3,7 @@
 // Started by Erik Hofman, February 2002
 // (Reuses some code from  fg_fx.cxx created by David Megginson)
 //
-// Copyright (C) 2002  Curtis L. Olson - curt@flightgear.org
+// Copyright (C) 2002  Curtis L. Olson - http://www.flightgear.org/~curt
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,7 +19,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: xmlsound.cxx,v 1.9 2004/09/08 11:15:57 ehofman Exp $
+// $Id: xmlsound.cxx,v 1.11 2004/11/21 17:05:42 ehofman Exp $
 
 #include <simgear/compiler.h>
 
@@ -80,11 +80,8 @@ SGXmlSound::~SGXmlSound()
 {
     _sample->stop();
 
-    if (_property)
-        delete _property;
-
-    if (_condition)
-        delete _condition;
+    delete _property;
+    delete _condition;
 
     _volume.clear();
     _pitch.clear();
